#!/bin/bash
# cleanup-jobdirs.sh

usage()
{
    exec 2>&1

    echo "Usage: $0 [options] [accounts]"
    echo "Options:"
    echo "	-c config_file"
    echo "	-d gridmapdir"
    echo "	-e   (name extra accounts to be cleaned up)"
    echo "	-h   (help)"
    echo "	-n   (dry run)"
    echo "	-t idle_time (in hours)"
    echo "	-v   (verbose)"

    exit 1
}

ACCOUNTS=
DRYRUN=false
EXTRA=
TIME=1
TIME_MIN=1
OPT_E=false
UID_MIN=100
VERBOSE=false
newline='
'

f=/etc/sysconfig/edg
test -f $f && . $f

DIR=${GRIDMAPDIR:-/etc/grid-security/gridmapdir}

f=/etc/sysconfig/lcg
test -f $f && . $f

CONF=${LCG_LOCATION:-/opt/lcg}/etc/cleanup-grid-accounts.conf	# sic!

for i
do
    case $i in
    -c)
	CONF=/dev/null
	break
    esac
done

test -f $CONF && . $CONF

while test $# != 0
do
    case $1 in
    -c)
	shift
	test $# = 0 && usage
	CONF=$1
	< $CONF || exit
	. $CONF
	;;
    -d)
	shift
	test $# = 0 && usage
	DIR=$1
	;;
    -e)
	OPT_E=true
	;;
    -h)
	usage
	;;
    -n)
	DRYRUN=true
	;;
    -t)
	shift
	test $# = 0 && usage
	TIME=$1
	;;
    -v)
	VERBOSE=true
	;;
    -*)
	echo "$0: unknown option: $1" >&2
	usage
	;;
    *)
	if $OPT_E
	then
	    EXTRA="$EXTRA $*"
	else
	    ACCOUNTS="$*"
	fi
	break
    esac

    shift
done

echo "0$TIME" | egrep '^0[0-9]+$' > /dev/null || usage

if test $TIME -lt $TIME_MIN
then
    echo "$0: ERROR: idle time $TIME < $TIME_MIN" >&2
    exit 1
fi

TIME=`expr "$TIME" '*' 60`

if test "x$ACCOUNTS" = x
then
    ACCOUNTS=`ls $DIR | egrep '^[^%]+[0-9]+$'`
fi

if test "x$EXTRA" != x
then
    ACCOUNTS="$ACCOUNTS $EXTRA"
fi

$VERBOSE && echo === Starting at `date`

for account in $ACCOUNTS
do
    uid=`id -u $account`

    if test "x$uid" = x
    then
	echo "$0: ERROR: skipping unknown user '$account'" >&2
	continue
    fi

    if test $uid -lt $UID_MIN
    then
	#
	# safety catch against accidental inclusion of root etc.
	#

	echo "$0: ERROR: skipping user '$account': UID $uid < $UID_MIN" >&2
	continue
    fi

    (
	eval dir=~$account/.globus/job

	[ -d $dir ] || exit
	cd $dir || exit

	cwd=`/bin/pwd`

	#
	# safety catches against somehow ending up in a system directory
	#

	case $cwd in
	/*/$account/.globus/job)
	    # could be OK
	    ;;
	*)
	    echo "$0: ERROR: $account: bad jobs directory $cwd" >&2
	    exit
	esac

	sysdirs='/(afs|bin|boot|dev|etc|initrd|lib|proc|root|sbin|usr)/'

	if echo "$cwd" | egrep "^$sysdirs" > /dev/null
	then
	    echo "$0: ERROR: $account: bad jobs directory $cwd" >&2
	    exit
	fi

	$VERBOSE && echo "Cleaning up $cwd"

	if $DRYRUN
	then
	    RMDIR='echo rmdir'
	else
	    RMDIR=rmdir
	fi

	$VERBOSE && LS=ls || LS=:
	$VERBOSE && LOG=: || LOG=echo

	perl -e '
	    $< = $> = '"$uid"';
	    exec(@ARGV) or die "$ARGV[0]: $\!\n";
	' \
	    find . -depth -mindepth 2 -type d -mmin +$TIME -user $account -exec \
		sh -c '
		    (ls "$0" | grep .) > /dev/null ||
		    ('"$LS"' -ld "$0"; '"$RMDIR"' "$0") ||
		    '"$LOG"' "could not rmdir: '"$cwd"'/$0" >&2
		' {} \;
    )
done

$VERBOSE && echo === Finished at `date`
exit 0

